
/* Portions Copyright (c) 1995,1996 by Trusted Information Systems, Inc.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND TRUSTED INFORMATION SYSTEMS DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL TRUSTED INFORMATION 
 * SYSTEMS BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 *
 * Trusted Information Systems, Inc. has received approval from the
 * United States Government for export and reexport of TIS/DNSSEC
 * software from the United States of America under the provisions of
 * the Export Administration Regulations (EAR) General Software Note
 * (GSN) license exception for mass market software.  Under the
 * provisions of this license, this software may be exported or
 * reexported to all destinations except for the embargoed countries of
 * Cuba, Iran, Iraq, Libya, North Korea, Sudan and Syria.  Any export
 * or reexport of TIS/DNSSEC software to the embargoed countries
 * requires additional, specific licensing approval from the United
 * States Government. 
 */




static char rcsid[] = "$Header: /u/h/ogud/sec_bind/dnssec_lib/RCS/rsaref_link.c,v 1.15 1996/09/17 20:40:24 ogud Exp $";
/* 
 * This file contains two components 
 * 1. Interface to the rsaref library to allow compilation of Bind 
 *    with TIS/DNSSEC when RSAREF is not available 
 *    all calls to RSAREF are contained inside this file.
 * 2. The glue to connvert RSA{REF} KEYS to and from external formats
 */
#include <stdio.h>
#include <unistd.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <sys/stat.h>
#include <errno.h>
#include "../conf/options.h"
#include "../include/arpa/nameser.h"
#include "../include/resolv.h"
#include "../conf/portability.h"
#include "../named/db_defs.h"
#include "../named/ns_defs.h"
#include "dnssec.h"
#include "dns_crypto.h"

#ifdef RSAREF
static void   InitRandomStruct  __P((R_RANDOM_STRUCT *));

/* 
 * rsa_sign() Function to call the appropriate RSAREF signing functions
 */
int 
rsa_sign( mode, context, data , len, priv_key, signature)
int mode, len;
R_SIGNATURE_CTX * context;
R_RSA_PRIVATE_KEY *priv_key;
u_char *data, *signature;
{
  int sign_len = 0;
  if( mode & SIG_MODE_INIT) 
    if( R_SignInit( context, DA_MD5))
      return SIGN_INIT_FAILURE;
      
  if( data && len > 0 ) /* equivalent of SIG_MODE_UPDATE */ 
    if( R_SignUpdate( context, data, len))
      return SIGN_UPDATE_FAILURE;
      
  if( mode & SIG_MODE_FINAL) 
    if( R_SignFinal( context, signature, &sign_len, priv_key))
      return SIGN_FINAL_FAILURE;
  return sign_len;
}

/* 
 * rsa_sign() Function to call the appropriate RSAREF verification functions
 */
rsa_verify( mode, context, data, len, pub_key, signature, sig_len)
int mode, len, sig_len;
R_SIGNATURE_CTX * context;
R_RSA_PUBLIC_KEY *pub_key;
u_char *data, *signature;
{
      if( mode & SIG_MODE_INIT) 
	if( R_VerifyInit( context, DA_MD5))
	  return VERIFY_INIT_FAILURE;
      
      if( data && len > 0 )
	if( R_VerifyUpdate( context, data, len))
	  return VERIFY_UPDATE_FAILURE;
      
      if( mode & SIG_MODE_FINAL)
	if( R_VerifyFinal( context, signature, sig_len, pub_key))
	  return VERIFY_FINAL_FAILURE;
      return 0;
}
/* 
 * rs2dnskey(): Function to convert key in RSAREF to DNS distribution format
 *         this function gets in a pointer to the public key and an work area
 *         to write the key into
 */
int
rsa2dns_key(public, out_str )
R_RSA_PUBLIC_KEY *public;
char *out_str;
{
  int n, loc;
  u_char *op,out[RAW_KEY_SIZE];
  op=out;
  if( public == NULL)
    return 0;
  for( n=0; public->exponent[n] == '\0'; n++);  /* find first non zero */
  *op++ = (u_char) MAX_RSA_MODULUS_LEN-n;      /* set lenght of exponent */
  for( ; n < MAX_RSA_MODULUS_LEN; op++)        /* copy exponent */
    *op = public->exponent[n++];
  loc = (public->bits+7)/8;                       /* find beg. of modulus */
  memcpy(op, &public->modulus[MAX_RSA_MODULUS_LEN-loc],loc);/*copy exponent*/
  n = res_encode64(out, out_str, out[0]+1+loc);      /* encode key */
  return n;
}
/* 
 * function that converts from a KEY RR format to a RSA KEY 
 * bzero() calls are required as malloc may return area with random data
 *
 */ 
void
dns2rsa_key( len, key, name, s_key)
int len;
u_char *key, *name;
KEY *s_key;
{
  int bytes;
  int i,j; 
  RSA_Key *r_key;
  if( key == NULL) 
    return;
  bytes = (int) key[0];
  i = len - bytes -1;
  r_key = (RSA_Key *) malloc( sizeof(struct rsakey));
  bzero(r_key, sizeof(KEY));
  s_key->KEY_struct = (void *) r_key; 
  s_key->signer = r_key->signer = (char *) strdup( name);
  r_key->Public_Key = (R_RSA_PUBLIC_KEY *) malloc(sizeof(R_RSA_PUBLIC_KEY));
  r_key->Private_Key = NULL;
  memset(r_key->Public_Key, 0, sizeof (R_RSA_PUBLIC_KEY));
  memcpy(&r_key->Public_Key->exponent[MAX_RSA_MODULUS_LEN-bytes],
	 &key[1], bytes);
  memcpy( &r_key->Public_Key->modulus[MAX_RSA_MODULUS_LEN-i],&key[bytes+1], i);
  r_key->Public_Key->bits = i * 8;
  s_key->foot = (u_int16_t) 
    _getshort( (u_char *) &r_key->Public_Key->modulus[MAX_RSA_MODULUS_LEN-3]);
  /* 
   * Here I should insert code to find first set bit and 
   *  decrement bits accordingly 
   */
#ifdef NOT_TESTED
  j = (u_int32_t) r_key->Public_Key->modulus[MAX_RSA_MODULUS_LEN-i];
  while(j) {
    j>>=1;
    r_key->Public_Key->bits--;
  }
#endif
  s_key->key_size = r_key->Public_Key->bits;
 return ;
}

/* function to free all dynamicly allocated structures in RSA_Key */
void
Free_RSAKey( r_key)
RSA_Key *r_key;
{
  if( r_key != NULL) {
    if(r_key->Private_Key)     SAFE_FREE(r_key->Private_Key);
    else if(r_key->Public_Key) SAFE_FREE(r_key->Public_Key);
    SAFE_FREE(r_key);
  }
  return ;
}

/* function to encode RSA Key into the portable file format */
int
Encode_RSAKey( rkey, buff, buff_len)
R_RSA_PRIVATE_KEY *rkey;
char *buff;
int buff_len;
{
  char *cp;
  int len;
  memset(buff, 0, buff_len);
  sprintf(buff,"Private-key-format: v%s\nAlgorithm: 1 (RSA)\n",
	  KEY_FILE_FORMAT);
  cp = (char *)strchr(buff,'\0');
  if((len=write_str(cp, "Modulus: ",rkey->modulus, 
		    MAX_RSA_MODULUS_LEN))<=0)    return -1;
  cp+=len;
  if ((len=write_str(cp, "PublicExponent: ",rkey->publicExponent, 
		     MAX_RSA_MODULUS_LEN))<=0)    return -2;
  cp += len;
  if ((len=write_str(cp, "PrivateExponent: ", rkey->exponent, 
		     MAX_RSA_MODULUS_LEN))<=0)   return -3;
  cp += len;
  if ((len=write_str(cp, "Prime1: ", rkey->prime[0], MAX_RSA_PRIME_LEN))<0) 
	 return -4;
  cp += len;
  if ((len=write_str(cp, "Prime2: ",rkey->prime[1], MAX_RSA_PRIME_LEN))<0)
    return -5;
  cp += len;
  if ((len=write_str(cp, "Exponent1: ",rkey->primeExponent[0], 
		     MAX_RSA_PRIME_LEN))<0) return -6;
  cp += len;
  if ((len=write_str(cp, "Exponent2: ",rkey->primeExponent[1], 
		     MAX_RSA_PRIME_LEN))<0) return -7;
  cp += len;
  if ((len=write_str(cp, "Coefficient: ",rkey->coefficient, 
           MAX_RSA_PRIME_LEN))<0) return -8;
  cp += len;
  return (int) (cp-buff);
}

int
Decode_RSAKey( r_key, buff, buff_len)
RSA_Key *r_key;
char *buff;
int buff_len;
{
  char *p = buff;
  R_RSA_PRIVATE_KEY key;

  int foot = -1;
  memset( &key, 0, sizeof(key));
  if (!verify_str(&p, "Modulus: ")) return -1;
  if (!read_str(&p, key.modulus, MAX_RSA_MODULUS_LEN)) return -2;
  key.bits = calculate_bits(key.modulus, MAX_RSA_MODULUS_BITS);
  while( *p && p < &buff[buff_len]) {
    if (verify_str(&p, "PublicExponent: ")) {
      if (!read_str(&p, key.publicExponent, MAX_RSA_MODULUS_LEN)) 
	return -3;
    }
    else if (verify_str(&p, "PrivateExponent: ")) {
      if (!read_str(&p, key.exponent, MAX_RSA_MODULUS_LEN)) 
	return -4;
    }
    else if (verify_str(&p, "Prime1: ")) {
      if (!read_str(&p, key.prime[0], MAX_RSA_PRIME_LEN)) 
	return -5;
    }
    else if (verify_str(&p, "Prime2: ")) {
      if (!read_str(&p, key.prime[1], MAX_RSA_PRIME_LEN)) 
	return -6;
    }
    else if (verify_str(&p, "Exponent1: ")){
      if (!read_str(&p, key.primeExponent[0], MAX_RSA_PRIME_LEN))
	return -7;
    }
    else if (verify_str(&p, "Exponent2: ")) {
      if (!read_str(&p, key.primeExponent[1], MAX_RSA_PRIME_LEN)) 
	return -8;
    }
    else if (verify_str(&p, "Coefficient: ")) {
      if (!read_str(&p, key.coefficient, MAX_RSA_PRIME_LEN)) 
	return -9;
    }
    else {
      printf("Decode_RSAKey(): Bad keyword %s\n", p);
      return -10;
    }
  }  /* while (p  */
  r_key->Private_Key = 
    (R_RSA_PRIVATE_KEY *) malloc( sizeof(R_RSA_PRIVATE_KEY));
  r_key->Public_Key = (R_RSA_PUBLIC_KEY *)  r_key->Private_Key ;
  memcpy( r_key->Private_Key, &key, sizeof(R_RSA_PRIVATE_KEY));
  foot = (int) (u_short) _getshort( 
	     &r_key->Public_Key->modulus[MAX_RSA_MODULUS_LEN-3]);
  return foot;
}


/*
 * Function used to generate new keys
 *
 * Generate_RSA_KEY() does all the necesary stuff to generate unique keys that
 * are hard to predict 
 * Parameters
 *   key  : generic Key structure
 *   exp  : the public exponent
 * Return 
 *     0 Failure 
 *     1 Success
 */
int 
Generate_RSAKey(key, exp)
    KEY *key;
     int exp;
{  
  R_RSA_PUBLIC_KEY *public;
  R_RSA_PRIVATE_KEY *private;
  R_RSA_PROTO_KEY proto;
  R_RANDOM_STRUCT randomStruct;
  RSA_Key *rsa;
  int status;

  public = (R_RSA_PUBLIC_KEY *) malloc(sizeof(R_RSA_PUBLIC_KEY));
  private = (R_RSA_PRIVATE_KEY *) malloc(sizeof(R_RSA_PRIVATE_KEY));
  proto.bits = key->key_size;
  proto.useFermat4 = exp ?  0x1 : 0x0;  /* 1 for f4=65537, 0 for f0=3 */
  printf("\nGen_RSA_KEY: Generating KEYS Please wait\n");
  InitRandomStruct(&randomStruct);
  status = R_GeneratePEMKeys(public, private, &proto, &randomStruct);
  if ( status) {
    printf("Gen_RSA_KEY:No Key Pair generated %d\n",status);
    free(public);
    free(private);
    return 0;
  }
  rsa = (RSA_Key *) malloc(sizeof(RSA_Key));
  rsa->signer      = key->signer;
  rsa->Private_Key = private;
  rsa->Public_Key  = public;
  key->KEY_struct  = (void *) rsa;
  return 1;
}

/*
 * This routine gets a bunch of system values to randomize the randomstruct
 * I use number of system calls to get somewhat unpredicable values
 * Then I call a special function prandom() that will do some magic depending
 * on the system used. 
 * If this function is executed on reasonably busy machine then the values
 * that prandom uses are hard to 
 *     1. Predict 
 *     2. Regenerate
 *     3. Hard to spy on as nothing is stored to disk and data is consumed
 *        as fast as it is generated.
 */
static void 
InitRandomStruct (randomstruct)
R_RANDOM_STRUCT *randomstruct;
{
  unsigned bytesNeeded;
  struct timeval tv;
  R_RandomInit (randomstruct);

  /* The runtime of the script is unpredictable within some range 
   * thus I'm getting the time of day again as this is an hard to guess 
   * value and the number of characters of the output from the script is
   * hard to guess. 
   * This must be the FIRST CALL 
   */
  gettimeofday(&tv, 0);
  R_RandomUpdate (randomstruct, (u_char *) &tv, sizeof(struct timeval));
  /* 
   * starting the script and performing stat() calls first adds 
   * unpredicatbility to the time gettimeofday is called
   * then we get some system parameters that an attacker may or may not know
   * it he/she has access to the system 
   */
  prandom( randomstruct, R_RandomUpdate);
  /* The runtime of the script is unpredictable within some range 
   * thus I'm getting the time of day again as this is an hard to guess 
   * value and the number of characters of the output from the script is
   * hard to guess. 
   * This must be the LAST CALL
   */
  gettimeofday(&tv, 0);
  R_RandomUpdate (randomstruct, (u_char *) &tv, sizeof(struct timeval));

  R_GetRandomBytesNeeded(&bytesNeeded, randomstruct);
  if (bytesNeeded) {
    printf("InitRandomStruct() didn't initialize enough randomness\n");
    exit(33);
  }
}
/*
 * key is the key to use 
 * [io]buf are the input and output buffers
 * flags selects weither  to use des program mode or hash mode 
 * mode ==0 is decrypt  ==1 is encrypt 
 */
int 
des_function( key, ibuf, obuf, len, mode)
u_char *key, *ibuf, *obuf;
int len, mode;
{
  DES_CBC_CTX context;  
  u_char zero[8];
  int i,v;
  v = i = 8-(len&0x7);
  memset(zero,0,sizeof(zero));

  DES_CBCInit( &context, key, zero, mode);
  v = 8-v;
  if( mode ==1) { /* mode 0 == decrypt  1 == encrypt */
    len += i;
    ibuf[len-1] = v; /* counts the number of bytes from this block to keep*/
  }
  DES_CBCUpdate( &context, obuf, ibuf, len);
  memset(&context,0, sizeof(context));
  return len;
}
#else  /* RSAREF */
int 
des_function ( key, ibuf, obuf, len, mode) 
u_char *key, *ibuf, *obuf;
int len, mode ;
{
  memcpy( obuf, ibuf, len);
  return len;
}
#endif /* RSAREF */
