
/* Portions Copyright (c) 1995,1996 by Trusted Information Systems, Inc.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND TRUSTED INFORMATION SYSTEMS DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL TRUSTED INFORMATION 
 * SYSTEMS BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 *
 * Trusted Information Systems, Inc. has received approval from the
 * United States Government for export and reexport of TIS/DNSSEC
 * software from the United States of America under the provisions of
 * the Export Administration Regulations (EAR) General Software Note
 * (GSN) license exception for mass market software.  Under the
 * provisions of this license, this software may be exported or
 * reexported to all destinations except for the embargoed countries of
 * Cuba, Iran, Iraq, Libya, North Korea, Sudan and Syria.  Any export
 * or reexport of TIS/DNSSEC software to the embargoed countries
 * requires additional, specific licensing approval from the United
 * States Government. 
 */




static char rcsid[] = "$Header: /m/a/projects/iip/iip/sec_bind494/dnssec_lib/RCS/support.c,v 1.6 1996/09/11 22:38:20 ogud Exp $";

#include <stdio.h>
#include <unistd.h>
#include <memory.h>
#include <string.h>
#include <errno.h>
#include <netinet/in.h>
#include "../conf/options.h"
#include "../include/arpa/nameser.h"
#include "../include/resolv.h"
#include "../conf/portability.h"
#include "ns_defs.h"
#include "db_defs.h"
#include "dnssec.h" 
#include "dns_crypto.h"

static int    ttyecho           __P((int, int));

/*
  Given a binary number stored as a u_char[], convert it to a base-64 string,
  and write it to the buffer.
*/
int 
write_str(buf, header, str, len) 
u_char *buf;
char *str, *header;
int len;
{
  char *p = str;
  int lenh,len64=0;
  while (!*p) p++,len--;
  if( header){
    strcpy( buf,header);
    lenh = strlen(buf);
  }
  len64 = res_encode64(p, buf+lenh, len);
  if (len64 < 0) return 0;
  buf[lenh+len64]='\n';
  return lenh+len64+1;
}


/*
  Validate that the string (str) is at the head of the buffer (buf),
  and if so, move the head of the buffer to after the string.
*/
int 
verify_str( buf, str)
char **buf, *str;
{
  int b,s;
  b=strlen(*buf);
  s=strlen(str);
  if (strncmp(*buf, str,s)) return 0;
  (*buf)+=s;
  return 1;
}

/*
  Read a base-64 encoded string from the head of the buffer until reaching
  a newline marker, convert it to binary, and save it in a char[].
  this procedure needs to worry about the network to host byte order
*/
int
read_str( buf, loc, loclen) 
char **buf;
u_char *loc;
int loclen;
{
  int len = strchr(*buf, '\n') - *buf;
  u_char bstr[RAW_KEY_SIZE]; /* too large but what the heck */
  int blen;
  if (len<=0) return 0;
  blen = res_decode64(*buf, bstr, len);
  if (blen<0) return 0;
  (*buf)+=(len+1);
  memset(loc, 0, loclen);
  memcpy(loc+loclen-blen, bstr, blen);
  return 1;
}

/*
  Given a binary number represented in a u_char[], determine the number of
  significant bits used.
*/
int calculate_bits(str, max_bits)
u_char *str;
int max_bits;
{
  u_char *p = str, i, j=0x80;
  int bits;
  for (bits=max_bits; *p==0x00; p++) 
    bits -=8;
  for(i = *p; i&j != j; j>>=1)
    bits--;
  return bits;
}

int rename_old( name, fmt_msg)
char *name, *fmt_msg;
{
  char tmp_name[512];
  if( access( name, R_OK)==0) { 
    sprintf( tmp_name, "%s.%s",name, "old_key");
    printf(fmt_msg,  tmp_name);
    if( rename(name, tmp_name)) {
      printf( "rename_old: Rename Falied %d\n", errno);
      return errno;
    }
  }
  return 0;
}
int 
get_passwd( pass, msg, pass_len)
u_char *pass;
char *msg;
int pass_len;
{
  int ti, o, i=0,cnt;
  u_char string[128], c;
  printf(msg);
  fflush(stdout);
  memset( string, 0, sizeof(string));
  ti = ttyecho(0,0);
  fgets( string, sizeof(string)-2, stdin);
  ti = ttyecho(0,ti);
  ti = strlen(string)-1;
  memset(pass, 0, pass_len);
  if( ti > 0) { /* des program compatability mode */
    strncpy(pass, string, 8);
    for(cnt = 0; cnt < 8; cnt++){  /* Set up out, determine parity bit */
      for(c = i = 0; i < 7 ; i++)
	if(pass[cnt] & (1<<i))
	  c++;
      if((c & 1) == 0)	pass[cnt] |=  0x80;
      else      	pass[cnt] &= ~0x80;
    }
  }
  /* erasing what ever was typed in */
  memset( string, 0, sizeof(string));
  putchar('\n');
  return ti;
}

/*
 * Function to turn echo on and off while reading passwords 
 */


#ifdef SYSV
#include <termios.h>
static int
ttyecho(fd, echo)
       int fd, echo;
{
       struct termios tio;
       int was_echo;

       if (!isatty(fd))
               return -1;
       tcgetattr(fd, &tio);
       was_echo = (tio.c_lflag & ECHO) != 0;
       if (echo)
               tio.c_lflag |= (ECHO|ECHONL);
       else
               tio.c_lflag &= ~(ECHO|ECHONL);
       tcsetattr(fd, TCSANOW, &tio);
       return was_echo;
}
#else
#include <sgtty.h>
static int
ttyecho(fd, echo)
int fd;
int echo;
{
    struct sgttyb ttyparams;
    int was_echo;

    if (!isatty (fd))
        was_echo = -1;
    else {
        ioctl(fd, TIOCGETP, &ttyparams);
        was_echo = (ttyparams.sg_flags & ECHO) != 0;
        if (echo)
            ttyparams.sg_flags = ttyparams.sg_flags | ECHO;
        else
            ttyparams.sg_flags = ttyparams.sg_flags & ~ECHO;
        ioctl(fd, TIOCSETP, &ttyparams);
    }

    return was_echo;
}
#endif /* SYSV */
