
/* Portions Copyright (c) 1995,1996 by Trusted Information Systems, Inc.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND TRUSTED INFORMATION SYSTEMS DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL TRUSTED INFORMATION 
 * SYSTEMS BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 *
 * Trusted Information Systems, Inc. has received approval from the
 * United States Government for export and reexport of TIS/DNSSEC
 * software from the United States of America under the provisions of
 * the Export Administration Regulations (EAR) General Software Note
 * (GSN) license exception for mass market software.  Under the
 * provisions of this license, this software may be exported or
 * reexported to all destinations except for the embargoed countries of
 * Cuba, Iran, Iraq, Libya, North Korea, Sudan and Syria.  Any export
 * or reexport of TIS/DNSSEC software to the embargoed countries
 * requires additional, specific licensing approval from the United
 * States Government. 
 */




/* $Header: /u/h/ogud/sec_bind/res/RCS/res_sign.c,v 1.6 1996/07/31 14:12:38 ogud Exp $ */

/*
 This file contains files required to encode and decode DNS data to and
 from base64 encoding. 
   out_len	= res_decode64(instr, outstr, instr_len)
   out_len	= res_encode64(instr, outstr, instr_len)
 */

#include <stdio.h>
#include <string.h>
#include <fcntl.h>

static unsigned char code_table[64] = {
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
	'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
	'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
	'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
	'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
	'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
	'w', 'x', 'y', 'z', '0', '1', '2', '3',
	'4', '5', '6', '7', '8', '9', '+', '/'
};

static unsigned char rev_table[256] = {
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,   62,  255,  255,  255,   63,
          52,   53,   54,   55,   56,   57,   58,   59,
/*        60,   61,  255,  255,  255,  255,  255,  255, */
          60,   61,  255,  255,  255,    0,  255,  255,  /* = == 0 */
         255,    0,    1,    2,    3,    4,    5,    6,
           7,    8,    9,   10,   11,   12,   13,   14,
          15,   16,   17,   18,   19,   20,   21,   22,
          23,   24,   25,  255,  255,  255,  255,  255,
         255,   26,   27,   28,   29,   30,   31,   32,
          33,   34,   35,   36,   37,   38,   39,   40,
          41,   42,   43,   44,   45,   46,   47,   48,
          49,   50,   51,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255,
         255,  255,  255,  255,  255,  255,  255,  255};

#define sextet_one(p)	((((p)[0]) >> 2) & 0x3f)
#define sextet_two(p)	(((((p)[0]) & 0x3) << 4) | ((((p)[1]) & 0xf0) >> 4))
#define sextet_three(p)	(((((p)[1]) & 0xf) << 2) | ((((p)[2]) >> 6) & 0x3))
#define sextet_four(p)	(((p)[2]) & 0x3f)

#define octet_one(p)	(((p)[0] << 2) | (((p)[1] >> 4) & 0x3))
#define octet_two(p)	((((p)[1] << 4) & 0xf0) | (((p)[2] >> 2) & 0xf))
#define octet_three(p)	(((((p)[2]) & 0x3) << 6) | ((p)[3]))

int 
res_encode64(instr, outstr, instr_len)
     u_char *instr, *outstr;
     int instr_len;
{
    int si, di; 
    if(instr  == NULL) return  0;  /* nothing to encode */
    if(instr_len < 0)  return -2;  /* nothing to encode */
    if(outstr == NULL) return -1;  /* no place to encode to */

    /* ENCODE ALL BUT LAST 1 or 2 BYTES unless multply of 3 */
    for (di = si = 0; si < instr_len-2; si+=3,di+=4) {
	outstr[di  ] = code_table[sextet_one(&(instr[si]))];
	outstr[di+1] = code_table[sextet_two(&(instr[si]))];
	outstr[di+2] = code_table[sextet_three(&(instr[si]))];
	outstr[di+3] = code_table[sextet_four(&(instr[si]))];
    }
    /* ENCODE LAST 1 or 2  BYTES */
    switch (instr_len-si) {
    case 1:
	outstr[di++] = code_table[sextet_one(&(instr[si]))];
	outstr[di++] = code_table[sextet_two(&(instr[si]))&0x30];
	outstr[di++] = '=';
        outstr[di++] = '=';
	break;
    case 2:
	outstr[di++] = code_table[sextet_one(&(instr[si]))];
	outstr[di++] = code_table[sextet_two(&(instr[si]))];
	outstr[di++] = code_table[sextet_three(&(instr[si]))&0x3c];
	outstr[di++] = '=';
	break;
    }
    outstr[di]='\0';
    return di;
  }



/***************************************************************************
 *  bdecode.c
 * This version of the routine returns the number bytes in decoded area
 *
 ***************************************************************************/

res_decode64( bin, bout, in_len)
	char           *bin;
	int            in_len;
        unsigned char   bout[];
{
	int             length, si, di, i;
	unsigned char   buf[4];

	/* DOUBLE CHECK PARAMETERS */
	if (bin == NULL)  return (-1);
	if (in_len == 0) return (0);   /* ZERO LENGTH INPUT BUFFER */

	/* CHECK THAT INPUT BUFFER LENGTH IS MULTIPLE OF 4 */
	if (in_len % 4 != 0) return (-2);

	/* Find lenght of OUTPUT */
	length = (in_len * 3) / 4;
	if (bin[in_len - 1] == '=') 	length--;
	if (bin[in_len - 2] == '=')	length--;

	/* DECODE ALL BUT LAST 4 BYTES */
	for (di = si = 0; si < in_len; si += 4, di += 3) {
		buf[0] = rev_table[bin[si]];
		buf[1] = rev_table[bin[si + 1]];
		buf[2] = rev_table[bin[si + 2]];
		buf[3] = rev_table[bin[si + 3]];
		bout[di] = octet_one(buf);
		bout[di + 1] = octet_two(buf);
		bout[di + 2] = octet_three(buf);
	}
	return length;
}


