
/* Portions Copyright (c) 1995,1996 by Trusted Information Systems, Inc.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND TRUSTED INFORMATION SYSTEMS DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL TRUSTED INFORMATION 
 * SYSTEMS BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 *
 * Trusted Information Systems, Inc. has received approval from the
 * United States Government for export and reexport of TIS/DNSSEC
 * software from the United States of America under the provisions of
 * the Export Administration Regulations (EAR) General Software Note
 * (GSN) license exception for mass market software.  Under the
 * provisions of this license, this software may be exported or
 * reexported to all destinations except for the embargoed countries of
 * Cuba, Iran, Iraq, Libya, North Korea, Sudan and Syria.  Any export
 * or reexport of TIS/DNSSEC software to the embargoed countries
 * requires additional, specific licensing approval from the United
 * States Government. 
 */




/* 
 * Started: 1995/1/29 
 * routines used for generating nxd records 
 * Extenaly visible routines 
 *        nxt_init()     ; resets number of variables
 *        nxt_addname()  ; adds name to list of names if name is there 
 *                       ; and/or updates the type bitmap for the name
 *        nxt_make()     ; Routine to generate NXT RR's for a given list 
 *                       ; of names
 *
 * $Header: /u/h/ogud/sec_bind/signer/RCS/nxt_util.c,v 1.25 1996/09/17 20:43:23 ogud Exp $
 */ 
#include <stdio.h>
#include <string.h> 
#include <memory.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/nameser.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <ctype.h>
#include <netdb.h>
#include "named.h"
#include "dnssec.h" 

static  int    nxt_make_RR     __P(( nxt *, nxt *, u_int32_t, u_int16_t,
				    struct databuf *));
static  nxt   *nxt_new         __P(( char *, int, char *, char *)),
              *nxt_find        __P(( char *, int));
static  char  *my_strdup       __P(( char *));

static int wild_len=0, signer_len=0, zone_len=0;
static char wild_name[MAXDNAME], nxt_signer[MAXDNAME], zone_name[MAXDNAME],
           *last=NULL;
static nxt *nxt_last=NULL;    
/* zone must be a valid string 
   signer can be a null string, in which case it the previous signer is 
        maintained
*/ 
void 
nxt_init(zone, signer) 
char *zone, *signer;
{
  nxt *nx, *nnx;
  if( zone == NULL || signer == NULL) 
    printf("nxt_init():Zone %s signer %s\n", zone, signer);
   /* I think this is the correct fix */
  if( zone == NULL)  
    panic(-1,"nxt_init():Zone name is required\n");

  if( (zone && strcasecmp(zone, zone_name) ==0) && 
      (signer && strcasecmp(signer, nxt_signer)==0))
    return ; /* repeat call ignore */
    
  for(nx=nxt_start; nx ; nx=nnx){ /* delete all elements in chain */
    nnx = nx->next;
    free(nx->dname);
    if( nx->signer && nx->signer!= nxt_signer) free(nx->signer);
    free(nx);
  }
  nxt_start = nxt_last = NULL;
  last = NULL;

  if( signer != NULL) { 
    /* save the name of the signer */
    signer_len = strlen(signer); 
    memset( nxt_signer, 0, sizeof(nxt_signer));
    if( signer_len < sizeof(nxt_signer)) 
      strncpy(nxt_signer, signer, signer_len);
    else {
      printf("nxt_util.c:nxt_init() name of signer too long %d >%d %s\n",
	     sizeof(nxt_signer), signer_len, signer);
      signer_len = 0;
    }
  }
  
  zone_len = strlen(zone);
  memset( zone_name, 0, sizeof(zone_name));
  if( zone_len < sizeof(zone_name)) 
    strncpy(zone_name, zone, zone_len);
  else {
    printf("nxt_util.c:nxt_init() name of zone too long %d >%d %s\n",
	   sizeof(zone_name), zone_len, zone);
    zone_len = 0;
  }
  memset(wild_name, 0, sizeof(wild_name));
  wild_len = 0;
}

/* 
 * nxt_addname(): Add a name to the nxt list if the name is not there already
 *        adds type to the nxt structure for that name
 *        uses global variables  do_not_sign, and sign_rec to
 *        direct if type is to be signed
 * Returns: 0 if everything is ok 
 *          
 */
int
nxt_addname(name,type, signer)
char *name, *signer; 
int  type;
{
  static nxt *nx,*t; 
  static char dname[MAXDNAME];
  int len, l,  j, dlen =0;
  /* first I need error checks */ 
  if ( type < 1 || type > 255) { 
    printf("nxt_addname(): Bad type %d\n",type);
    return NXT_BAD_TYPE;
  }
  /* is this name in the right zone */
  len = strlen(name); 
  dlen = len - zone_len;
  /* strong name checking on additons to NXT except for Key files */
  if( dlen < 0 || 
     (key_file==0 && strncasecmp( zone_name, &name[dlen],zone_len))){
    printf("nxt_util.c:nxt_addname() REJECT Name not in zone %s %s\n",
	   name, zone_name);
    return NXT_NOT_IN_ZONE; 
  }
  /* get the domain name */
  if( dlen > 0)  strncpy( dname, name, dlen--);
  dname[dlen]=0x0;
/* now check if it is in a subzone */
  if( dlen > 0 && strchr(dname, '.') != NULL) { 
    printf("nxt_util.c:nxt_addname() REJECT Name from subzone %s\n",name);
    return NXT_SUBZONE_NAME;
  }
/*  printf("nxt_util.c:nxt_addname() NAME %s\n", dname);  */
  if(( last == NULL) || (strcasecmp(last, dname) != 0)) { /*  ignore case */
    if( dname[0] == '*' && wild_len == 0) { /* wild card name */ 
      wild_len = strlen(name); 
      strncpy(wild_name, name, wild_len);
    }
    nx = nxt_find(dname, dlen);
    if( nx == NULL) {       /* This name sorts to the beginning of the list */
      nx = nxt_new(dname, dlen, name, signer); 
      nx->next = nxt_start;
      if(nxt_start == NULL) nxt_last = nx;
      nxt_start = nx;
    }
    else {
      l = dlen < nx->len ? dlen : nx->len; /* I want smaller one */
      j = strncasecmp(dname, nx->dname,l); 
      if( (j > 0) || ((dlen > nx->len) && ( j==0))) { /* new name */
	t = nxt_new(dname, dlen, name, signer);
	t->next = nx->next;
	if( nx == nxt_last) nxt_last = t;  /* move last pointer to new one */
	nx = nx->next = t; 
      }
      last = nx->dname;
    }   
  }
  /* check if signer has been declared */
  if( (nx->signer == NULL && signer != NULL)) 
    nx->signer = savestr(signer);
/* NXT mask shows the all known types */
  if (type > nx->max_type)
    nx->max_type = type;
  SIGN_BIT_SET(type, nx->nxt_rr);
/* mark record to be signed, 
   if name has signer, sign_rec flags is set and do_not_sign   flag is not set 
   or this is a NXT record 
 */
  if( nx->signer && !do_not_sign && sign_rec) {
    (void) dprintf(10,(ddt, "nxt_addname: %s %d\n", name,type));
    SIGN_BIT_SET( type, nx->sign_rr);
    if (type > nx->max_sig)
      nx->max_sig = type;
  }
  return 0;
}

/* I need to duplicate string and leave enough space in it to append
   a dot to it */
static char *
my_strdup(name) 
char *name;
{
  int len = strlen(name);
  char *out = malloc(len+2);
  strncpy(out, name,len);
  out[len++]='\0';
  out[len]='\0';
  return out;
}

/* 
 * nxt_new() returns a new nxt_data structure partialy filled out with default
 * values.
 */ 
static nxt *
nxt_new( dname, len, full_name, signer)
char *dname, *signer, *full_name;
int len;
{
  struct nxt_struct *tmp; 
  tmp = (nxt *) malloc(sizeof(nxt));
  memset( tmp, 0, sizeof( nxt));
  tmp->len   =  len;
  tmp->dname = my_strdup( dname);
  tmp->name =  my_strdup( full_name);
  if( signer && strcmp( nxt_signer, signer)==0) 
    tmp->signer = nxt_signer;
  else if( signer == NULL) 
    tmp->signer = NULL;
  else { /* trying to declare signer other than nxt_signer this is an error */
    printf("nxt_new(): declaring bad signer= %s  expecting %s\n", signer,
	   nxt_signer);
    tmp->signer = my_strdup( signer);
  }
  return tmp;
}

/* 
 * nxt_find(): returns the pointer to the name if found 
 * or the lexigraphicly one in front of it
 */ 
static nxt *
nxt_find(name, len) 
char *name; 
int len;
{
  nxt *curr, *prev=NULL; 
  int j; 
/* fast search to approx location  
 *  Change to skip list or Array lookup later XXXX 
 */
  /* first quick check if the name would be appened at the end */
  if( nxt_last && len > 0 && strcasecmp( name, nxt_last->dname) > 0)  {
    return nxt_last;
  }  
  for( curr=nxt_start; curr && curr->dname[0] < name[0]; curr=curr->next)
    prev = curr; 
/* Now I got close by the name start checking */
  while( curr && curr->dname[0]==name[0]){  /* while name starts with letter */
    j = strcasecmp(name, curr->dname);
    if( j==0 && len == curr->len)     /*if lenght is the same check if */
      return curr;                    /* same name */
    else if( (j < 0) ||(( j==0) && ( len > curr->len)) )  
      return prev; 
    prev = curr;
    curr = curr->next;
  }
  return prev; 
}

/* 
 * nxt_make_RR(): Function to generate an NXT record for a name
 *    this NXT RR is added to the database. 
 */ 
static int
nxt_make_RR( curr_ptr, next_ptr , ttl, sec, xdp)
nxt * curr_ptr, *next_ptr;
struct databuf *xdp;
#ifdef NOT_NEEDED
void *xdp;
#endif NOT_NEEDED
u_int32_t ttl;
u_int16_t sec;
{

  struct databuf *dp;
  int dbflags = DB_NODATA;
  char wrk_area[MAXDNAME*4]; 
  int out_size=0, maxl, c;

/*  printf ("nxt_make_RR(): %s NXT %s\n",curr_ptr->dname, next_ptr->dname); */
  if( (hide && wild_len > 0) || curr_ptr->dname[0]=='*') 
      memcpy(wrk_area, wild_name, out_size=wild_len);
  else {
    if( next_ptr->len){
      strncpy(wrk_area, next_ptr->dname, out_size=next_ptr->len);
      wrk_area[out_size++]='.';
    }
    strncpy(&wrk_area[out_size], zone_name, zone_len); 
    out_size += zone_len;
  }
  wrk_area[out_size++] = '\0'; 
  
  /* 
   * set NXT and SIG bits in masks
   */
  SIGN_BIT_SET(T_NXT, curr_ptr->nxt_rr); /* set NXT flag */
  if (curr_ptr->max_type < T_NXT)
    curr_ptr->max_type =  T_NXT;
  if(  sec > 0 /* || curr_ptr->max_sig > 0*/) { 
    SIGN_BIT_SET(T_SIG, curr_ptr->nxt_rr); /* set SIG flag */
    SIGN_BIT_SET(T_NXT, curr_ptr->sign_rr); /* set NXT flag */
    SIGN_BIT_SET(T_SIG, curr_ptr->sign_rr); /* set SIG flag */
    if (curr_ptr->max_sig < T_NXT)
      curr_ptr->max_sig =  T_NXT;
  } 
  maxl = (curr_ptr->max_type+7)/8;
  memcpy( &wrk_area[out_size], curr_ptr->nxt_rr, maxl);
  out_size += maxl;
  {
    const char *fname;
    struct hashbuf *htp=hashtab;
    struct namebuf *np;
    np = nlookup( curr_ptr->name, &htp, &fname, 0);
    if( np ==NULL) { 
      dprintf(10, ("nxt_make_RR(): No such name %s\n", curr_ptr->name));
      return 0;
    }
    xdp = np->n_data;
  }
  dp = savedata( C_IN, T_NXT, ttl, wrk_area, out_size);
  dp->d_zone =  xdp->d_zone;
  dp->d_flags = xdp->d_flags;
  dp->d_cred =  xdp->d_cred; 
  dp->d_clev =  xdp->d_clev;
  dp->d_sec  =  xdp->d_sec;  
  c = db_update(curr_ptr->name, dp, dp, dbflags, hashtab);
  if( c  != OK) {
    printf("nxt_make_RR():update failed %s type=%d update code %d \n", 
	   curr_ptr->dname, T_NXT ,c);
    free((char*) dp);
    return 0;
  }
  return out_size;
}

/* 
 * nxt_make(): This function generates NXT RR's based on the names
 * in the list provided 
 * Returns the number of NXT records generated
 */ 
int
nxt_make( name, ttl, sec)
char *name;
u_int16_t sec;
u_int32_t ttl;
{
  nxt *curr;
  struct databuf *xdp=NULL;
  int pnx_size;
  const char *fname;
  int no=0;
  if( strcasecmp(name,zone_name)) {
    printf("Requested wrong zone %s Current zone %s\n", name, zone_name);
    return (-1);
  }
  if( nxt_start != NULL) {
    struct hashbuf *htp=hashtab;
    struct namebuf *np;
    np = nlookup( zone_name, &htp, &fname, 0);
    xdp = np->n_data;
    for (curr=nxt_start; curr->next; curr =curr->next, no++) {
      if( hide || (curr->next->dname[0] != '*')) 
	pnx_size = nxt_make_RR(curr, curr->next, ttl, sec, xdp);
      else if( curr->next->next) /* skip over wild card name */
	pnx_size = nxt_make_RR(curr, curr->next->next, ttl, sec, xdp);
      else  /* if no record after wild card point to end of zone */
	pnx_size = nxt_make_RR(curr, nxt_start, ttl, sec, xdp);
    }
    pnx_size = nxt_make_RR( curr, nxt_start, ttl, sec, xdp);
    no++;
  } /* if */
  return no;
}
